#include "stm32f4xx_hal.h"
#include "stm3240G-eval.h"
#include "demo.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#define STARTTASK_PRIO 3
#define STARTTASK_DELAY 100
#define STARTTASK_STACKSIZE 128

#define LEDTASK1_PRIO 3
#define LEDTASK1_DELAY 1000
#define LEDTASK1_STACKSIZE 128

#define LEDTASK2_PRIO 4
#define LEDTASK2_DELAY 55
#define LEDTASK2_STACKSIZE 128

static void LEDblink1(void *pvParameters );
static void LEDblink2(void *pvParameters );
static void StartTask(void *pvParameters );


SemaphoreHandle_t  LEDMutex;

#ifdef TRC_USE_TRACEALYZER_RECORDER
traceString LED1Chn;
#endif


int main(void)
{
    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
    HAL_Init();
    /* Configure the system clock */
    SystemClock_Config();
    /* Initialize all configured peripherals */
    MX_GPIO_Init();

//#ifdef TRC_USE_TRACEALYZER_RECORDER
//#if ((TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_SNAPSHOT)||  (! USE_ITM_STREAMING)) 
//    vTraceEnable(TRC_START);
//#else
//    vTraceEnable(TRC_INIT); 
//#endif  
//#endif
    vTraceEnable(TRC_START);
    
    LEDMutex = xSemaphoreCreateMutex();
    
#if (TRC_USE_TRACEALYZER_RECORDER == 1) 
    vTraceSetSemaphoreName(LEDMutex,"LED_Mutex");
    LED1Chn = xTraceRegisterString("LED1Status"); 
#endif  

	xTaskCreate( StartTask, "Startup", STARTTASK_STACKSIZE, NULL, STARTTASK_PRIO, NULL );


    vStartDemoApplication();



    /* Start scheduler */
    vTaskStartScheduler();
    while (1)
    {
    }
}

static void StartTask(void *pvParameters )
{
    /* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
    

    xTaskCreate( LEDblink1, "LED1", LEDTASK1_STACKSIZE, NULL, LEDTASK1_PRIO, NULL );
	xTaskCreate( LEDblink2, "LED2", LEDTASK2_STACKSIZE, NULL, LEDTASK2_PRIO, NULL );
    

    for(;;)
    {
        vTaskDelay(STARTTASK_DELAY);
        SetLED(LED4,LED_ON);
        vTaskDelay(STARTTASK_DELAY);
        SetLED(LED4,LED_OFF);        
    }
}

static void LEDblink1(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
    
    uint32_t * mem;
   
    while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        SetLED(LED1,LED_ON);
        mem=pvPortMalloc(4);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK1_DELAY);
        SetLED(LED1,LED_OFF);
        vPortFree(mem);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK1_DELAY);    
	}
}

static void LEDblink2(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
	while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        SetLED(LED1,LED_ON);
    	#if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK2_DELAY);
        SetLED(LED1,LED_OFF);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
     		vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK2_DELAY);   
	}
}


